# AssistedKills release 1 (oy1) by David Bowland
# ./addons/eventscripts/assistedkills/assistedkills.py

# >>> To configure this addon please see assistedkills.cfg <<<

"""
Fires a custom event when a player kills an opponent but did not cause the most damage to that opponent. Requires ES 2.0+
Custom event: player_assisteddeath
"""


import es
import langlib
import os.path
import playerlib


info = es.AddonInfo()
info.name = 'AssistedKills'
info.version = '1'
info.url = 'http://addons.eventscripts.com/addons/view/assistedkills'
info.basename = 'assistedkills'
info.author = 'SuperDave'


dict_players = {}
dict_player_count = {}

dict_events = {'player_assisteddeath':{'userid':'short', 'attacker':'short', 'assistid':'short', 'weapon':'string', 'damage':'short', 'attackerdamage':'short', 'assistcount':'short', 'assistedcount':'short'}}
dict_options = {'assistedkills_threshold':[15, 'Amount of damage a player must have more than the killing player to receive an assisted kill'], 'assistedkills_reward_money':[150, 'Money a player receives for an assisted kill'], 'assistedkills_reward_killstopoints':[2, 'Number of assisted kills a player needs to receive points added to his or her score'], 'assistedkills_reward_points':[1, 'Number of points a player will receive for the requisite number of assisted kills'], 'assistedkills_punishment_money':[150, 'Money a player loses for a kill that was assisted'], 'assistedkills_punishment_killstopoints':[5, 'Number of kills that can be assisted until player loses points from his or her score'], 'assistedkills_punishment_points':[2, 'Number of points a player will lose for the requisite number of kills that are assisted'], 'assistedkills_announce_console':[2, '0 = no change, 1 = announce assisted kills to involved players on the console, 2 = announce assisted kills to all players on the console'], 'assistedkills_announce_center':[2, '0 = no change, 1 = announce assisted kills to the player who made the assisted kill in center text, 2 = announce assisted kills to players involved in the assisted kill in center text'], 'assistedkills_announce_chat':[0,'0 = no change, 1 = announce assisted kills to the player who made the assisted kill in chat area, 2 = announce assisted kills to players involved in the assisted kill in chat area, 3 = announce assisted kills to all players in chat area']}
func_lang_text = None


def load():
   """
   Checks to make sure critical server variables are created by the config
   Calls player_spawn for each player
   """
   global func_lang_text

   for str_option in dict_options:
      es.ServerVar(str_option, dict_options[str_option][0], dict_options[str_option][1])
   if os.path.isfile(es.getAddonPath('assistedkills') + '/assistedkills.cfg'):
      es.server.cmd('es_xmexec ../addons/eventscripts/assistedkills/assistedkills.cfg')
   else:
      es.dbgmsg(0, 'AssistedKills: Unable to load assistedkills.cfg! Please ensure it is in the ./assistedkills/ directory.')

   str_lang_path = es.getAddonPath('assistedkills') + '/assistedkills_languages.ini'
   if os.path.isfile(str_lang_path):
      func_lang_text = langlib.Strings(str_lang_path)
   else:
      es.dbgmsg(0, 'AssistedKills: Unable to load assistedkills_languages.ini! Please ensure it is in the ./assistedkills/ directory.')
   if not func_lang_text:
      func_lang_text = missing_lang

   load_events()

   for int_userid in es.getUseridList():
      player_spawn({'userid':str(int_userid)})


def es_map_start(event_var):
   """
   Calls load_events
   Clears dict_players and dict_player_count so data is recreated each map
   """
   load_events()

   dict_players.clear()
   dict_player_count.clear()


def player_spawn(event_var):
   """
   Creates or clears the spawning player of damage data
   Creates the count of assist kills and assisted kills if necessary
   """
   global dict_players
   global dict_player_count

   str_userid = event_var['userid']
   if dict_players.has_key(str_userid):
      dict_players[str_userid].clear()
   else:
      dict_players[str_userid] = {}

   if not dict_player_count.has_key(str_userid):
      dict_player_count[str_userid] = {'assists':0, 'assisted':0}


def player_hurt(event_var):
   """Stores the amount of damage each player takes and from whom"""
   global dict_players

   str_attacker = event_var['attacker']
   if int(str_attacker) > 0:
      if dict_players[event_var['userid']].has_key(str_attacker):
         dict_players[event_var['userid']][str_attacker] += int(event_var['dmg_health'])
      else:
         dict_players[event_var['userid']][str_attacker] = int(event_var['dmg_health'])


def player_disconnect(event_var):
   """Removes the disconnecting player from dict_players and dict_player_count"""
   global dict_players
   global dict_player_count

   str_userid = event_var['userid']
   if dict_players.has_key(str_userid):
      del dict_players[str_userid]
   if dict_player_count.has_key(str_userid):
      del dict_player_count[str_userid]


def player_death(event_var):
   """
   Finds the player who caused the most damage to the dead player
   Adds an assist kill to the assisting player and requisite rewards
   Adds an assisted kill to the player who killed the victim and the requisite punishments
   Fires the player_assisteddeath event
   Displays announcements to players
   """
   global dict_player_count

   str_userid = event_var['userid']
   dict_current_player = dict_players[str_userid]
   int_highest_damage = 0
   str_assist_userid = 0
   str_attacker = event_var['attacker']
   dict_current_player['0'] = 0

   for str_loop_userid in dict_current_player:
      int_loop_damage = dict_current_player[str_loop_userid]
      if int_loop_damage > int_highest_damage:
         int_highest_damage = int_loop_damage
         str_assist_userid = str_loop_userid

   if not dict_current_player.has_key(str_attacker):
      dict_current_player[str_attacker] = 0

   if int_highest_damage and str_assist_userid <> str_attacker and dict_current_player[str_attacker] + es.ServerVar('assistedkills_threshold') < int_highest_damage:

      if event_var['es_userteam'] <> str(es.getplayerteam(str_assist_userid)) and dict_player_count.has_key(str_assist_userid):
         dict_player_count[str_assist_userid]['assists'] += 1

         if int(es.ServerVar('assistedkills_reward_killstopoints')) and int(es.ServerVar('assistedkills_reward_points')):
            if not dict_player_count[str_assist_userid]['assists'] % es.ServerVar('assistedkills_reward_killstopoints'):
               apply_score(str_assist_userid, int(es.ServerVar('assistedkills_reward_points')))

         if int(es.ServerVar('assistedkills_reward_money')):
            int_money = es.getplayerprop(str_assist_userid, 'CCSPlayer.m_iAccount') + es.ServerVar('assistedkills_reward_money')
            if int_money > 16000:
               int_money = 16000
            es.setplayerprop(str_assist_userid, 'CCSPlayer.m_iAccount', int_money)

      if event_var['es_userteam'] <> event_var['es_attackerteam'] and int(str_attacker):
         dict_player_count[str_attacker]['assisted'] += 1

         if int(es.ServerVar('assistedkills_punishment_killstopoints')) and int(es.ServerVar('assistedkills_punishment_points')):
            if not dict_player_count[str_attacker]['assists'] % es.ServerVar('assistedkills_punishment_killstopoints'):
               apply_score(str_attacker, int(es.ServerVar('assistedkills_punishment_points')))

         if int(es.ServerVar('assistedkills_punishment_money')):
            int_money = es.getplayerprop(str_attacker, 'CCSPlayer.m_iAccount') - es.ServerVar('assistedkills_punishment_money')
            if int_money < 0:
               int_money = 0
            es.setplayerprop(str_attacker, 'CCSPlayer.m_iAccount', int_money)

      es.event('initialize', 'player_assisteddeath')
      es.event('setint', 'player_assisteddeath', 'userid', str_userid)
      es.event('setint', 'player_assisteddeath', 'attacker', str_attacker)
      es.event('setint', 'player_assisteddeath', 'assistid', str_assist_userid)
      es.event('setstring', 'player_assisteddeath', 'weapon', event_var['weapon'])
      es.event('setint', 'player_assisteddeath', 'damage', int_highest_damage)
      es.event('setint', 'player_assisteddeath', 'attackerdamage', dict_current_player[str_attacker])
      if dict_player_count.has_key(str_assist_userid):
         es.event('setint', 'player_assisteddeath', 'assistcount', dict_player_count[str_assist_userid]['assists'])
      else:
         es.event('setint', 'player_assisteddeath', 'assistcount', 0)
      if int(str_attacker):
         es.event('setint', 'player_assisteddeath', 'assistedcount', dict_player_count[str_attacker]['assisted'])
      else:
         es.event('setint', 'player_assisteddeath', 'assistedcount', 0)
      es.event('fire', 'player_assisteddeath')

      str_assist_name = es.getplayername(str_assist_userid)
      dict_assist_tokens = {'assist':str_assist_name, 'victim':event_var['es_username'], 'damage':int_highest_damage}
      int_option = int(es.ServerVar('assistedkills_announce_console'))
      if int_option:
         if int_option == 2:
            for int_loop_userid in es.getUseridList():
               es.cexec(int_loop_userid, 'echo ' + remove_tags(func_lang_text('long text', dict_assist_tokens, playerlib.getPlayer(int_loop_userid).get('lang'))))
         else:
            for str_loop_userid in (str_userid, str_attacker, str_assist_userid):
               if int(str_loop_userid):
                  es.cexec(str_loop_userid, 'echo ' + remove_tags(func_lang_text('long text', dict_assist_tokens, playerlib.getPlayer(str_loop_userid).get('lang'))))

      int_option = int(es.ServerVar('assistedkills_announce_center'))
      if int_option:
         es.centertell(str_assist_userid, remove_tags(func_lang_text('assisted kill', {'victim':event_var['es_username']}, playerlib.getPlayer(str_assist_userid).get('lang'))))
         if int_option == 2:
            if int(str_attacker):
               es.centertell(str_attacker, remove_tags(func_lang_text('kill assisted', {'assist':str_assist_name}, playerlib.getPlayer(str_attacker).get('lang'))))
            es.centertell(str_userid, remove_tags(func_lang_text('death assisted', {'assist':str_assist_name}, playerlib.getPlayer(str_userid).get('lang'))))

      int_option = int(es.ServerVar('assistedkills_announce_chat'))
      if int_option:
         if int_option <> 3:
            es.tell(str_assist_userid, '#multi', func_lang_text('assisted kill', {'victim':event_var['es_username']}, playerlib.getPlayer(str_assist_userid).get('lang')))
         if int_option == 2:
            if int(str_attacker):
               es.tell(str_attacker, '#multi', func_lang_text('kill assisted', {'assist':str_assist_name}, playerlib.getPlayer(str_attacker).get('lang')))
            es.tell(str_userid, '#multi', func_lang_text('death assisted', {'assist':str_assist_name}, playerlib.getPlayer(str_userid).get('lang')))
         elif int_option == 3:
            for int_loop_userid in es.getUseridList():
               es.tell(int_loop_userid, '#multi', func_lang_text('long text', dict_assist_tokens, playerlib.getPlayer(int_loop_userid).get('lang')))


def unload():
   for str_option in dict_options:
      es.ServerVar(str_option).set(0)


def apply_score(str_userid, int_score):
   """
   Ensures there is one game_score entity
   Adds the points to the player's score
   """
   dict_game_scores = es.createentitylist('game_score')
   int_game_score_index = es.getentityindex('game_score')

   if len(dict_game_scores) > 1:
      for int_index in dict_game_scores:
         if int_index <> int_game_score_index:
            es.server.cmd('es_xremove %s' % int_index)
   elif int_game_score_index == -1:
      es.server.cmd('es_xgive %s game_score' % str_userid)

   es.server.cmd('es_xfire %s game_score addoutput \"spawnflags 1\"' % str_userid)
   es.server.cmd('es_xfire %s game_score addoutput \"points %s\"' % (str_userid, int_score))
   es.server.cmd('es_xfire %s game_score applyscore' % str_userid)


def load_events():
   """Loads the .res file for the player_assisteddeath event"""
   if os.path.isfile(es.getAddonPath('assistedkills') + '/assistedkills.res'):
      es.loadevents('declare', 'addons/eventscripts/assistedkills/assistedkills.res')
   else:
      if not os.path.isfile(es.getAddonPath('assistedkills') + 'es_assistedkills_events_db.txt'):
         es.keygroupcreate('assistedkills_events')
         for str_event in dict_events:
            es.keycreate('assistedkills_events', str_event)
            for str_event_var in dict_events[str_event]:
               es.keysetvalue('assistedkills_events', str_event, str_event_var, dict_events[str_event][str_event_var])
         es.keygroupsave('assistedkills_events', '|assistedkills')
         es.keygroupdelete('assistedkills_events')
      es.loadevents('declare', 'addons/eventscripts/assistedkills/es_assistedkills_events_db.txt')
      es.dbgmsg(0, 'AssistedKills: Unable to load assistedkills.res! Please ensure it is in the ./assistedkills/ directory.')


def remove_tags(str_text):
   """Removes #lightgreen, #green and #default tags from the supplied string"""
   return str_text.replace('#lightgreen', '').replace('#green', '').replace('#default', '')


def missing_lang(str_heading, dict_tokens, str_language):
   """Displays an error message because the languages .ini could not be found"""
   return 'AssistedKills: assistedkills_languages.ini not found'